package org.imixs.workflow.jee.rest.servlet;

import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This class is a handler for a RestService URI. The class expects the servlet
 * request and response object and also a ServletContext
 * The URIHandler parses the requestURI and supports a set of properties 
 * @author rsoika
 * 
 */
public class URIHandler {
	private HttpServletRequest request;
	private HttpServletResponse response;
	private String mainresource = "";
	private String subresource = "";
	private String file = "";
	private String download = "";
	private String options = "";
	private String format = "";
	private String xsl = "";
	private String contenttype = "";
	private String encoding = "";
	private int count = -1;
	private int startpos = 0;
	private ServletContext servletContext;
	
	private static final List<String> SUPPORTED_FORMAT_EXTENTIONS = Arrays.asList("xml", "xsl", "txt", "html", "xhtml");  
	
	public URIHandler(HttpServletRequest arequest,
			HttpServletResponse aresponse, ServletContext sc) {
		this.request = arequest;
		this.response = aresponse;
		this.servletContext = sc;
		parseURI();
	}

	public HttpServletRequest getRequest() {
		return request;
	}

	public HttpServletResponse getResponse() {
		return response;
	}

	public String getMainresource() {
		return mainresource;
	}

	public String getSubresource() {
		return subresource;
	}

	public String getFile() {
		return file;
	}

	public String getDownload() {
		return download;
	}

	public void setDownload(String download) {
		this.download = download;
	}

	public String getOptions() {
		return options;
	}

	public String getFormat() {
		return format;
	}

	public String getXsl() {

		return xsl;
	}

	public int getCount() {
		return count;
	}

	public int getStartpos() {
		return startpos;
	}

	public String getContenttype() {
		return contenttype;
	}

	public String getEncoding() {
		if (encoding == null || "".equals(encoding))
			encoding = "UTF-8";

		return encoding;
	}

	public boolean hasEncoding() {
		return (encoding != null && !"".equals(encoding));
	}

	/**
	 * returns true if the mainresource is of typ workflow
	 * 
	 * @return
	 */
	public boolean isWorkflowRequest() {
		return "workflow".equals(mainresource);
	}

	/**
	 * returns true if the mainresource is of typ workflow
	 * 
	 * @return
	 */
	public boolean isModelRequest() {
		return "model".equals(mainresource);
	}

	/**
	 * returns true if the mainresource is of typ workflow
	 * 
	 * @return
	 */
	public boolean isReportRequest() {
		return "report".equals(mainresource);
	}

	/**
	 * returns true if the subresource is one of the collection resources
	 * 
	 * @return
	 */
	public boolean isCollectionRequest() {
		// parse valid view
		if (subresource != null && !"".equals(subresource)) {
			if (("worklist".equals(subresource))
					|| ("statuslist".equals(subresource))
					|| ("worklistbycreator".equals(subresource))
					|| ("worklistbygroup".equals(subresource))
					|| ("worklistbyprocessid".equals(subresource))
					|| ("worklistbyref".equals(subresource)))
				return true;

		}
		return false;
	}

	public boolean isFileRequest() {
		return ((!"".equals(file)));
	}

	/**
	 * This method parses the URI and stores the different parts in member
	 * variables. The vars consits of a ResourceName (required) a optional
	 * UniqueID, optionals OutputFormat and optional a $file field. e.g.
	 * worklist/xxxxxxx.xml?count=10&start=10
	 */
	private void parseURI() {
		mainresource = "";

		options = "";
		count = -1;
		startpos = 0;

		// WorkList??
		String sPath = request.getPathInfo();
		if (sPath == null)
			return;

		StringTokenizer stringtok = new StringTokenizer(sPath, "/");

		/*
		 * The first part (Token) need to be one of the main resources
		 * 'workflow', 'report' or 'model'
		 */

		if (stringtok.hasMoreTokens())
			mainresource = stringtok.nextToken().toLowerCase();

		// verify main resource
		if ((!"workflow".equals(mainresource))
				&& (!"model".equals(mainresource))
				&& (!"report".equals(mainresource)))
			return; // invalid request!

		// parse subresource
		if (stringtok.hasMoreTokens())
			subresource = stringtok.nextToken().toLowerCase();

		/* Check for $file */
		if (stringtok.hasMoreTokens()) {
			// expected format /[filename]
			file = stringtok.nextToken();
		}

		/*
		 * now parse representation Format if provided the format option is
		 * always a .xxx extension at the end of the subresource
		 * Supported are ".txt" ".xml" ".xsl" ".html" ".xhtml"
		 */
		format = "xml";
		if (subresource.lastIndexOf(".") > -1) {
			format = subresource.substring(subresource.lastIndexOf(".") + 1);
			// test if format is a supported file extension
			if (SUPPORTED_FORMAT_EXTENTIONS.contains(format)) {
				// cut extension
				subresource = subresource
						.substring(0, subresource.lastIndexOf("."));
			} else {
				// no valid extension so reset format to default xml
				format="xml";
			}
 				
			
		} else {
			// if the request is a report with no Extension than the format will
			// default to "xslt"
			if ("report".equals(mainresource))
				format = "xslt";
		}

		/*
		 * Now parse query options
		 */
		options = request.getQueryString();
		if (options != null) {

			// parse Count
			if (options.toLowerCase().indexOf("count=") > -1) {
				String sCount = options.substring(options.toLowerCase()
						.indexOf("count=") + 6);
				if (sCount.indexOf("&") > -1)
					sCount = sCount.substring(0, sCount.indexOf("&"));
				try {
					count = Integer.parseInt(sCount);
				} catch (NumberFormatException nfe) {
					count = -1;
				}

			}

			// parse Startpos
			if (options.toLowerCase().indexOf("start=") > -1) {
				String sPos = options.substring(options.toLowerCase().indexOf(
						"start=") + 6);
				if (sPos.indexOf("&") > -1)
					sPos = sPos.substring(0, sPos.indexOf("&"));
				try {
					startpos = Integer.parseInt(sPos);
				} catch (NumberFormatException nfe) {
					startpos = 0;
				}

			}

			// xsl template?
			if (options.toLowerCase().indexOf("xsl=") > -1) {
				String sPos = options.substring(options.toLowerCase().indexOf(
						"xsl=") + 4);
				if (sPos.indexOf("&") > -1)
					sPos = sPos.substring(0, sPos.indexOf("&"));
				xsl = sPos;
				// convert file location...
				xsl = servletContext.getRealPath(xsl);

			}

			// content type?
			if (options.toLowerCase().indexOf("contenttype=") > -1) {
				String sPos = options.substring(options.toLowerCase().indexOf(
						"contenttype=") + 12);
				if (sPos.indexOf("&") > -1)
					sPos = sPos.substring(0, sPos.indexOf("&"));
				contenttype = sPos;

			}

			// encoding?
			if (options.toLowerCase().indexOf("encoding=") > -1) {
				String sPos = options.substring(options.toLowerCase().indexOf(
						"encoding=") + 9);
				if (sPos.indexOf("&") > -1)
					sPos = sPos.substring(0, sPos.indexOf("&"));
				encoding = sPos;

			}

			// download filename?
			if (options.toLowerCase().indexOf("download=") > -1) {
				String sPos = options.substring(options.toLowerCase().indexOf(
						"download=") + 9);
				if (sPos.indexOf("&") > -1)
					sPos = sPos.substring(0, sPos.indexOf("&"));
				download = sPos;

			}

		} // end of sOptions!=null

	}

}
